const SerialPort = require('serialport');

const config = require('komodo-sdk/config');
const logger = require('komodo-sdk/logger');

const ParserInterByteTimeout = require('@serialport/parser-inter-byte-timeout');

const parsers = require('./lib/serialport-parsers');
const modemCommands = require('./lib/modem-commands');
const modemInfo = require('./lib/modem-info');

const parserInterByteTimeout = new ParserInterByteTimeout({ interval: 1000 });
parserInterByteTimeout.on('data', (data) => {
    logger.verbose('INCOMING', { parser: 'parserInterByteTimeout', data: data.toString() });
});

const port = new SerialPort(config.modem.device, { baudRate: 115200 }, async (err) => {
    if (err) {
        logger.warn(`Error opening modem. ${err}. Terminating modem ${config.modem.device}.`);
        process.exit(1);
    }

    await modemCommands.sendCtrlZ();
    // await modemCommands.writeToPortAndWaitForOkOrError('\rAT\r');
    await modemCommands.writeToPortAndWaitForOkOrError('AT&FE0\r');
    await modemCommands.writeToPortAndWaitForOkOrError('AT+CMGF=0\r');
    await modemCommands.writeToPortAndWaitForOkOrError('AT+CNMI=1,2,0,1,0\r');

    await modemCommands.queryManufacturer();
    await modemCommands.queryModel();

    await modemCommands.queryIMEIAndIMSI();
    await modemCommands.queryCOPSAndSignalQuality();
    logger.info('Modem state', modemInfo);

    if (config.debug_sms_destination_on_start) {
        await modemCommands.sendSMS(config.debug_sms_destination_on_start, `${modemInfo.imsi || 'UNKNOWN'}: coba pakai pdu ${new Date()}`);
    }

    if (config.debug_ussd_code_on_start) {
        const ussdResponse = await modemCommands.executeUSSD(config.debug_ussd_code_on_start, 2);
        logger.info('USSD RESPONSE', { command: config.debug_ussd_code_on_start, ussdResponse });
    }

    setInterval(async () => {
        await modemCommands.initATCommands();
        await modemCommands.queryManufacturer();
        await modemCommands.queryModel();
        await modemCommands.queryIMEIAndIMSI();
        await modemCommands.queryCOPSAndSignalQuality();
        logger.info('Modem state', modemInfo);
    }, config.interval_beetwen_signal_strength_ms || 30000);
});

global.MODEM_PORT = port;
parsers.setPort(port);
// modemCommands.setPort(port);

if (config && config.modem_tester && config.modem_tester.parser === 'regex') {
    logger.info('Using parserWaitForOkOrError');
    port.pipe(parsers.parserWaitForOkOrError);
} else if (config && config.modem_tester && config.modem_tester.parser === 'interbyte') {
    logger.info('Using parserInterByteTimeout');
    port.pipe(parserInterByteTimeout);
} else {
    logger.info('Using parserReadline');
    port.pipe(parsers.parserReadline);
}
